import * as hooks from '../src/index';

// Build object with all imports
const hooksObj = { ...hooks };

const supportedSyntax = [
  '<length>',
  '<number>',
  '<percentage>',
  '<length-percentage>',
  '<color>',
  '<image>',
  '<url>',
  '<integer>',
  '<angle>',
  '<time>',
  '<resolution>',
  '<transform-function>',
  '<custom-ident>',
  '<transform-list>',
];

const supportedTypes = ['any', 'color', 'number', 'string', 'raw'];
const supportedCategories = ['color', 'font', 'spacing', 'radius', 'sizing', 'shadow'];

/* eslint-env jest */
describe('sds-styling-hooks', () => {
  it('all props have a namespace', async () => {
    Object.keys(hooksObj).map((category) => {
      const hooks = hooksObj[category];
      if (hooks.global) {
        expect(hooks.global.namespace).toBe('sds');
      }
    });
  });
  it('all props have a scope of global or shared', async () => {
    Object.keys(hooksObj).map((category) => {
      const hooks = hooksObj[category];
      if (hooks.global.scope) {
        expect(hooks.global.scope === 'global' || hooks.global.scope === 'shared').toBeTruthy();
      } else {
        Object.keys(hooks.props).map((key) => {
          const prop = hooks.props[key];
          expect(prop.scope === 'global' || prop.scope === 'shared').toBeTruthy();
        });
      }
    });
  });
  it('all props have a boolean inherit value', async () => {
    Object.keys(hooksObj).map((category) => {
      const hooks = hooksObj[category];
      if (hooks.global.inherits) {
        expect(typeof hooks.global.inherits).toEqual('boolean');
      } else {
        Object.keys(hooks.props).map((key) => {
          const prop = hooks.props[key];
          expect(typeof prop.inherits).toEqual('boolean');
        });
      }
    });
  });
  it('all props have a type value', async () => {
    Object.keys(hooksObj).map((category) => {
      const hooks = hooksObj[category];
      if (hooks.global.type) {
        expect(supportedTypes).toContain(hooks.global.type);
      } else {
        Object.keys(hooks.props).map((key) => {
          const prop = hooks.props[key];
          expect(supportedTypes).toContain(prop.type);
        });
      }
    });
  });
  it('all props contain one of the supported syntax names', async () => {
    Object.keys(hooksObj).map((category) => {
      const hooks = hooksObj[category];
      if (hooks.global.type) {
        expect(supportedSyntax).toContain(hooks.global.syntax);
      } else {
        Object.keys(hooks.props).map((key) => {
          const prop = hooks.props[key];
          expect(supportedSyntax).toContain(prop.syntax);
        });
      }
    });
  });
  it('all props contain one of the supported category name', async () => {
    Object.keys(hooksObj).map((category) => {
      const hooks = hooksObj[category];
      if (hooks.global.category) {
        expect(supportedCategories).toContain(hooks.global.category);
      } else {
        Object.keys(hooks.props).map((key) => {
          const prop = hooks.props[key];
          expect(supportedCategories).toContain(prop.category);
        });
      }
    });
  });
  it('all props contain a usage comment', async () => {
    Object.keys(hooksObj).map((category) => {
      const hooks = hooksObj[category];
      Object.keys(hooks.props).map((key) => {
        const prop = hooks.props[key];
        expect(prop.comment.length).toBeGreaterThan(0);
      });
    });
  });
  it('if hex value is defined, check that the type is set to "color"', async () => {
    Object.keys(hooksObj).map((category) => {
      const hooks = hooksObj[category];
      const hexReg = /(#[A-Fa-f0-9]{6}|#[A-Fa-f0-9]{3})/;
      Object.keys(hooks.props).map((key) => {
        const prop = hooks.props[key];
        const hex = hexReg.test(prop.value);
        if (hex === true) {
          expect(prop.type).toEqual('color');
        }
      });
    });
  });
});
